from __future__ import print_function
from __future__ import unicode_literals
import paho.mqtt.client as mqtt
from coapthon.client.helperclient import HelperClient
import json

class MqttSession(object):
    def __init__(self, token, host, port=1883, keepalive=60):
        self.host = host
        self.token = token
        self.port = port
        self.keepalive = 60
        self.client = mqtt.Client()
        self.client.username_pw_set(token)

class RestHttpSession(object):
    def __init__(self, device_id, host, jwt_token):
        self.device_id = device_id
        self.host = host
        self.token = jwt_token

class AdminRestSession(object):
    def __init__(self, host, jwt_token):
        self.host = host
        self.jwt_token = jwt_token


class DeviceHttpSession(object):
    pass

class CoAPSession(object):
    def __init__(self, host, token, port=5683, timeout=1):
        self.host = host
        self.token = token
        self.port = port
        self.timeout = timeout



from __future__ import print_function
from __future__ import unicode_literals
from .session import MqttSession, CoAPSession
from coapthon.client.helperclient import HelperClient
import json

class Mqtt(MqttSession):
    def upload(self, payload):
        self.client.connect(self.host, self.port, self.keepalive)
        self.client.publish("v1/devices/me/telemetry", json.dumps(payload), 1)

class CoAP(CoAPSession):
    def post(self, payload):
        path = "/api/v1/{token}/telemetry".format(token=self.token)
        client = HelperClient(server=(self.host, self.port))
        response = client.post(path, json.dumps(payload))
        print(response.pretty_print())
        client.stop()
        return response

class MqttGateway(MqttSession):
    def upload(self, payload):
        self.client.connect(self.host, self.port, self.keepalive)
        self.client.publish("v1/gateway/telemetry", json.dumps(payload), 1)



from __future__ import print_function
from __future__ import unicode_literals
import paho.mqtt.client as mqtt
from .session import MqttSession
import requests
import json

class Session(MqttSession):
    # #response when rpc from server side
    # def response(self, msg, payload):
    #     self.client.publish(msg.topic.replace('request', 'response'), json.dumps(payload), 1)

    #rpc from server side 
    def listen_request(self):
        self.client.connect(self.host, self.port, self.keepalive)

        def on_connect(client, userdata, rc, *extra_params):
            self.client.subscribe('v1/gateway/rpc')
            
            if hasattr(self, 'user_on_connect_action'):
                self.user_on_connect_action(rc)
           
            print("Start to listen Server-side RPC")

        def on_message(client, userdata, msg):
            print('Received Topic: ', msg.topic, 'Message: ', str(msg.payload))
            
            if hasattr(self, 'user_on_message_action'):
                self.user_on_message_action(msg)

        self.client.on_connect = on_connect
        self.client.on_message = on_message

        try:
            self.client.loop_forever()
        except KeyboardInterrupt:
            print("Connection Closed")

if __name__ == "__main__":
    s = Session(host='192.168.1.71', token='AU700o0tKRNby9sdAwCZ')

    def on_message(msg):
        print("on message")

    def on_connect(rc):
        print("on connect")

    s.user_on_message_action = on_message
    s.user_on_connect_action = on_connect
    s.listen_request()
    



from setuptools import setup, find_packages

setup(
    name="aihub-things",
    packages=find_packages(),
    version='0.5.0',
    description="aihub things sdk",
    author="zouqqz",
    author_email='zouqqz@gmail.com',
    keywords=['sdk', 'aihub'],
    url='http://github.com',
    install_requires=['paho-mqtt', 'requests', 'CoAPthon'],
)



from __future__ import print_function
from __future__ import unicode_literals
import paho.mqtt.client as mqtt
from coapthon.client.helperclient import HelperClient
from .session import MqttSession, CoAPSession
import requests
import json

class Session(CoAPSession):
    def observe(self, callback=None, timeout=None):
        path = "/api/v1/{token}/rpc".format(token=self.token)

        def default_callback(response):
            print(response.pretty_print())

        if callback == None:
            callback = default_callback

        client = HelperClient(server=(self.host, self.port))
        client.observe(path, callback, timeout)

    def post(self, payload):
        path = "/api/v1/{token}/rpc".format(token=self.token)
        client = HelperClient(server=(self.host, self.port))
        response = client.post(path, json.dumps(payload))
        print(response.pretty_print())
        client.stop()
        return response


from __future__ import print_function
from __future__ import unicode_literals
from .session import MqttSession, CoAPSession
from coapthon.client.helperclient import HelperClient
import json

class Mqtt(MqttSession):
    def upload(self, payload):
        self.client.connect(self.host, self.port, self.keepalive)
        self.client.publish("v1/devices/me/attributes", json.dumps(payload), 1)

    def request(self, payload):
        self.client.connect(self.host, self.port, self.keepalive)

        def on_connect(client, userdata, rc, *extra_params):
            self.client.subscribe('v1/devices/me/attributes/response/+')
            self.client.publish('v1/devices/me/attributes/request/1', json.dumps(payload))

        def on_message(client, userdata, msg):
            print('Received Topic: ', msg.topic, 'Message: ', str(msg.payload))
            self.client.disconnect()
            self.msg = msg

        self.client.on_connect = on_connect
        self.client.on_message = on_message
        self.client.loop_forever()
        return self.msg

    def shared_subscribe(self, payload):
        self.client.connect(self.host, self.port, self.keepalive)
        self.client.subscribe('v1/devices/me/attributes')
        self.client.loop_forever()

class CoAP(CoAPSession):
    def post(self, payload):
        path = "/api/v1/{token}/attributes".format(token=self.token)
        client = HelperClient(server=(self.host, self.port))
        response = client.post(path, json.dumps(payload))
        print(response.pretty_print())
        client.stop()
        return response

    def get(self, attrs, shared=[]):
        path = "/api/v1/{token}/attributes?clientKeys={attrs}&sharedKeys={shared}".format(
            token=self.token, attrs=','.join(attrs), shared=','.join(shared))
        client = HelperClient(server=(self.host, self.port))
        response = client.get(path)
        print(response.pretty_print())
        client.stop()
        return response

    def observe(self, callback=None, timeout=None, **kwargs):
        path = "/api/v1/{token}/attributes".format(token=self.token)

        def client_callback_observe(response):  # pragma: no cover
            print("Callback_observe")

        client = HelperClient(server=(self.host, self.port))
        client.observe(path, client_callback_observe, timeout, *kwargs)


from __future__ import print_function
from __future__ import unicode_literals
from .session import RestHttpSession
import requests
import json

class Session(RestHttpSession):
    def request(self, payload, type=1):
        type = "oneway" if type == 1 else "twoway"
        requst_url = "http://{host}/api/plugins/rpc/{type}/{id}".format(host=self.host,  type=type, id=self.device_id)
        headers = {"X-Authorization": "Bearer " + self.token}
        r = requests.post(requst_url, json=payload, headers=headers)

        if type=="oneway":
            return r.status_code
        else:
            return r.status_code, r.text


from __future__ import print_function
from __future__ import unicode_literals
from .session import AdminRestSession
import requests
import json

class Session(AdminRestSession):
    def save_device(self, payload, gateway_token, device_type):
        requst_url = "http://{host}/device/save".format(host=self.host)
        payload['jwtToken'] = self.jwt_token
        payload['gatewayToken'] = gateway_token
        payload['deviceType'] = device_type
        r = requests.post(requst_url, json=payload)
        return r.text

    def delete_device(self, payload, gateway_token):
        requst_url = "http://{host}/device/delete".format(host=self.host)
        payload['jwtToken'] = self.jwt_token
        payload['gatewayToken'] = gateway_token
        r = requests.post(requst_url, json=payload)
        return r.text

    def save_relation(self, payload, gateway_token, device_type):
        requst_url = "http://{host}/relation/save".format(host=self.host)
        payload['jwtToken'] = self.jwt_token
        payload['gatewayToken'] = gateway_token
        payload['deviceType'] = device_type
        r = requests.post(requst_url, json=payload)
        return r.text